#pragma once
#ifndef PLAYER_MOVEMENT_GR_H
#define PLAYER_MOVEMENT_GR_H

#include <chrono>
#include <thread>
#include <iostream>
#include "sgg/graphics.h"
#define M_PI 3.14159265358979323846

// Ιδιότητες και κατάσταση του παίκτη
struct Player {
    float x;             // Θέση του παίκτη στον άξονα x
    float y;             // Θέση του παίκτη στον άξονα y
    float width, height; ///< Διαστάσεις του παίκτη (πλάτος και ύψος).
    float speed;         // Τρέχουσα ταχύτητα του παίκτη
    float angle;         ///< Προσανατολισμός του παίκτη σε μοίρες.
    bool isJumping;      // Υποδεικνύει αν ο παίκτης εκτελεί άλμα
    bool isCrouching;    // Υποδεικνύει αν ο παίκτης είναι σκυμμένος
    const float defaultHeight = 1.0f;  // Προεπιλεγμένο ύψος για την όρθια θέση
    const float crouchHeight = 0.5f;   // Ύψος όταν ο παίκτης είναι σκυμμένος

    graphics::Brush frontBrush; ///< Χρώμα που χρησιμοποιείται για το εμπρόσθιο τμήμα του παίκτη.
    graphics::Brush backBrush;
};

/**
 * Καθυστερεί την απόκριση του πληκτρολογίου για συγκεκριμένο χρονικό διάστημα.
 * Η συνάρτηση αυτή σταματά την εκτέλεση του προγράμματος για να προσομοιώσει
 * καθυστέρηση στην απόκριση του πληκτρολογίου, αγνοώντας ουσιαστικά
 * εισόδους από το πληκτρολόγιο κατά τη διάρκεια της καθυστέρησης.
 *
 * @param delay_ms Διάρκεια καθυστέρησης σε χιλιοστά του δευτερολέπτου πριν αποδεχτεί εισόδους.
 *
 * @note Αυτή η συνάρτηση μπορεί να χρησιμοποιηθεί για να περιορίσει τη συχνότητα εισόδων σε περιπτώσεις
 * όπου οι γρήγορες αποκρίσεις δεν είναι επιθυμητές.
 */
void slowKeyboardInput(int delay_ms) {
    if (delay_ms > 0) {
        std::this_thread::sleep_for(std::chrono::milliseconds(delay_ms));
    }
}

/**
 * Κάνει τον παίκτη να εκτελέσει άλμα στον άξονα y.
 * @param player Αναφορά στη δομή Player.
 * @param jumpHeight Το ύψος του άλματος.
 * Αυτή η συνάρτηση ενημερώνει τη θέση του παίκτη στον άξονα y
 * αυξάνοντας τη συντεταγμένη y για την προσομοίωση άλματος.
 */
void jumpY(Player& player, float jumpHeight) {
    if (!player.isJumping) {
        player.y += jumpHeight;
        player.isJumping = true;
        std::cout << "Ο παίκτης εκτέλεσε άλμα στον άξονα Y στη θέση: " << player.y << std::endl;
    }
    player.isJumping = false;
}

/**
 * Κάνει τον παίκτη να εκτελέσει άλμα στον άξονα x.
 * @param player Αναφορά στη δομή Player.
 * @param jumpDistance Η απόσταση του άλματος.
 * Αυτή η συνάρτηση ενημερώνει τη θέση του παίκτη στον άξονα x
 * αυξάνοντας τη συντεταγμένη x για την προσομοίωση άλματος.
 */
void jumpX(Player& player, float jumpDistance) {
    if (!player.isJumping) {
        player.x += jumpDistance;
        player.isJumping = true;
        std::cout << "Ο παίκτης εκτέλεσε άλμα στον άξονα X στη θέση: " << player.x << std::endl;
    }
    player.isJumping = false;
}

/**
 * Αυξάνει την ταχύτητα του παίκτη για μικρό χρονικό διάστημα, προσομοιώνοντας σπριντ.
 * @param player Αναφορά στη δομή Player.
 * @param sprintMultiplier Πολλαπλασιαστής που εφαρμόζεται στην ταχύτητα του παίκτη.
 * Η συνάρτηση εφαρμόζει προσωρινή αύξηση της ταχύτητας, προσφέροντας στον παίκτη
 * ώθηση για πιο γρήγορη κίνηση.
 */
void sprint(Player& player, float sprintMultiplier) {
    player.speed *= sprintMultiplier;
    std::cout << "Ο παίκτης σπριντάρει με ταχύτητα: " << player.speed << std::endl;
}

/**
 * Μειώνει την ταχύτητα κίνησης του παίκτη για πιο αθόρυβη προσέγγιση.
 * @param player Αναφορά στη δομή Player.
 * @param walkMultiplier Πολλαπλασιαστής που εφαρμόζεται για τη μείωση της ταχύτητας του παίκτη.
 * Η συνάρτηση αυτή επιβραδύνει την ταχύτητα του παίκτη, επιτρέποντας προσεκτικότερη κίνηση.
 */
void walk(Player& player, float walkMultiplier) {
    player.speed *= walkMultiplier;
    std::cout << "Ο παίκτης περπατά με ταχύτητα: " << player.speed << std::endl;
}

// Συναρτήσεις μετακίνησης και περιστροφής

/**
 * Μετακινεί τον παίκτη προς τα εμπρός προς την κατεύθυνση που κοιτάει.
 * @param player Ο παίκτης που θα μετακινηθεί προς τα εμπρός.
 */
void moveForward(Player& player) {
    player.x += player.speed * cos(player.angle * M_PI / 180.0f);
    player.y += player.speed * sin(player.angle * M_PI / 180.0f);
}

/**
 * Μετακινεί τον παίκτη προς τα πίσω προς την αντίθετη κατεύθυνση που κοιτάει.
 * @param player Ο παίκτης που θα μετακινηθεί προς τα πίσω.
 */
void moveBackward(Player& player) {
    player.x -= player.speed * cos(player.angle * M_PI / 180.0f);
    player.y -= player.speed * sin(player.angle * M_PI / 180.0f);
}

/**
 * Περιστρέφει τον παίκτη προς τα αριστερά κατά συγκεκριμένες μοίρες.
 * @param player Ο παίκτης που θα περιστραφεί.
 * @param degrees Η ποσότητα περιστροφής σε μοίρες.
 */
void rotateLeft(Player& player, float degrees) {
    player.angle -= degrees;
    if (player.angle < 0) player.angle += 360;
}

/**
 * Περιστρέφει τον παίκτη προς τα δεξιά κατά συγκεκριμένες μοίρες.
 * @param player Ο παίκτης που θα περιστραφεί.
 * @param degrees Η ποσότητα περιστροφής σε μοίρες.
 */
void rotateRight(Player& player, float degrees) {
    player.angle += degrees;
    if (player.angle >= 360) player.angle -= 360;
}

// Συναρτήσεις ελέγχου θέσης και ορίων

/**
 * Ελέγχει αν ο παίκτης βρίσκεται εντός των ορίων του παραθύρου.
 * @param player Ο παίκτης που θα ελεγχθεί.
 * @param windowWidth Το πλάτος του παραθύρου.
 * @param windowHeight Το ύψος του παραθύρου.
 * @return true αν ο παίκτης είναι εντός των ορίων, αλλιώς false.
 */
bool isWithinBounds(const Player& player, float windowWidth, float windowHeight) {
    return player.x >= player.width / 2 && player.x <= windowWidth - player.width / 2 &&
        player.y >= player.height / 2 && player.y <= windowHeight - player.height / 2;
}

/**
 * Διασφαλίζει ότι ο παίκτης παραμένει εντός των ορίων του παραθύρου.
 * @param player Ο παίκτης για τον οποίο θα εφαρμοστούν τα όρια.
 * @param windowWidth Το πλάτος του παραθύρου.
 * @param windowHeight Το ύψος του παραθύρου.
 */
void enforceBounds(Player& player, float windowWidth, float windowHeight) {
    if (player.x < player.width / 2) player.x = player.width / 2;
    if (player.x > windowWidth - player.width / 2) player.x = windowWidth - player.width / 2;
    if (player.y < player.height / 2) player.y = player.height / 2;
    if (player.y > windowHeight - player.height / 2) player.y = windowHeight - player.height / 2;
}

// Συναρτήσεις γραφικής απεικόνισης

/**
 * Σχεδιάζει τον παίκτη ως ορθογώνιο με διακριτή μπροστινή και πίσω πλευρά.
 * Ο προσανατολισμός του παίκτη λαμβάνεται υπόψη, με ελαφριά μετατόπιση
 * για να διακρίνεται οπτικά η μπροστινή πλευρά.
 * @param player Ο παίκτης που θα σχεδιαστεί.
 */
void drawPlayer(const Player& player) {
    graphics::setOrientation(player.angle); // Θέτει τον προσανατολισμό σύμφωνα με τη γωνία του παίκτη

    // Ορισμός χρώματος για την πίσω πλευρά
    graphics::Brush backBrush = player.backBrush;
    backBrush.fill_color[0] = 0.6f;
    backBrush.fill_color[1] = 0.6f;
    backBrush.fill_color[2] = 0.6f;

    // Ορισμός χρώματος για την μπροστινή πλευρά
    graphics::Brush frontBrush = player.frontBrush;
    frontBrush.fill_color[0] = 1.0f;
    frontBrush.fill_color[1] = 0.8f;
    frontBrush.fill_color[2] = 0.0f;

    // Σχεδίαση πίσω πλευράς
    graphics::drawRect(player.x, player.y, player.width, player.height, backBrush);

    // Σχεδίαση μπροστινής πλευράς με ελαφριά μετατόπιση
    float frontOffset = player.width * 0.2f;
    float frontX = player.x + frontOffset * cos(player.angle * M_PI / 180.0f);
    float frontY = player.y + frontOffset * sin(player.angle * M_PI / 180.0f);
    graphics::drawRect(frontX, frontY, player.width * 0.8f, player.height * 0.8f, frontBrush);

    graphics::resetPose(); // Επαναφορά προσανατολισμού μετά τη σχεδίαση
}

#endif // PLAYER_MOVEMENT_GR_H
